<?php

// This filter looks for text including markup and inserts bibliogaphic references
// Based on bibliography module from drupal (biblio by Ron Jerome)

/**
* @author Susana Leito <sleitao at reit dot up dot pt>
* @param        int            course id
* @param        string         text to be filtered
*/

function bibliography_filter($courseid, $text) {
    global $CFG, $COURSE;

    if (!is_string($text)) {
        // non string data can not be filtered anyway
        return $text;
    }
    $sql = "SELECT *
        FROM mdl_modules AS modules, mdl_course_modules AS cmodules
        WHERE modules.id = cmodules.module
        AND cmodules.course = $COURSE->id
        AND modules.name = 'bibliography'";

    if (get_records_sql($sql)){ // only if there is a bibliography module in course
        $newtext = bibref($text);
        return $newtext;
    }
    else return $text;
}

/**
* Finds matching <bib>citekey</bib> and calls the filter function
* Returns the filtered text (inline refs and footer with bibliography entries)
*
* @param string $text The text to be filtered
*/
function bibref($text){
    //$pattern = array('|\[bib](.*?)\[/bib]|s', '|<bib>(.*?)\</bib>|s');
    $pattern = '|\[bib-(.*?)\]|s';
    $text = preg_replace_callback($pattern, '_biblio_filter_replace_callback', $text);
    $footer = '';
    $footer = _biblio_filter_replace_callback(NULL, 'output footer');
    if (preg_match('/<bibliography(\/( )?)?>/', $text) > 0) {
        $text = preg_replace('/<bibliography(\/( )?)?>/', $footer, $text, 1);
        return $text;
    }
    else {
        return $text ."\n\n". $footer;
    }
}

/**
* Filter function
* Returns the filtered text (inline citations and footer with bibliography entries)
*
* @param array $matches
* @param string $op
*/
function _biblio_filter_replace_callback($matches, $op = '') {
    static $n = 0;
    static $store_matches = array();
    $str = '';
    /// output footer
    if ($op == 'output footer') {
        if ($n > 0) {
            $str = '<p><hr /></p><h3 id="references">'.get_string('filter_footerreferences','bibliography').'</h3>';
            $str .= '<div class="references"><ol>';
            for ($m = 1; $m <= $n; $m++) {
                $str .= '<li id="reference'. $m .'">'. _biblio_citekey_print($store_matches[$m -1]) ." </li>\n\n";
            }
            $str .= '</ol></div>';
        }
        $n = 0;
        $store_matches = array();
        return $str;
    }

    //default op: act as called by preg_replace_callback()
    if ($matches[1]!=""){ // if there is something between bib tag
        if (($ref = array_search(trim_brackets($matches[1]),$store_matches))===FALSE) {
            $n++;
            array_push($store_matches, trim_brackets($matches[1]));
            $ref = $n;
        } else { 
            $ref++;
        }
        $allowed_tags = array();
        $title = filter_xss($matches[1], $allowed_tags);
        //html attribute cannot contain quotes
        $title = str_replace('"', "&quot;", $title);
        //remove newlines. Browsers don't support them anyway and they'll confuse line break converter in filter.module
        $title = str_replace("\n", " ", $title);
        $title = str_replace("\r", "", $title);
        $text = '<span hovertip="reference'. $ref .'">[<a href="#reference'.$ref.'">'. $ref .'</a>]</span>';
        return $text;
    }
}

/**
* Prints footer with bibliography references
* Returns printed references
*
* @param string $citekey
*/
function _biblio_citekey_print($citekey) {
    global $CFG, $COURSE;
    $authors = array();
    if ( (strrpos($citekey, "[")) || (strrpos($citekey, "]")) ){ // if you are using square brackets in citation
        $citekey = trim_brackets($citekey); // trim the brackets
    }
    //Get bibliography entry that corresponds to the citekey from all bibliography entries from the present course
    $sql = "SELECT be.id, be.year, be.last_update_date, be.title, be.context_title, be.edition, be.editor, be.location, be.publisher, be.volume, be.issue, be.pages, be.doi, be.access_date, be.url, be.bibliographyid
        FROM {$CFG->prefix}bibliography_entries AS be
        JOIN {$CFG->prefix}bibliography AS b ON (be.bibliographyid = b.id)
        WHERE be.citekey=\"$citekey\" AND b.course=$COURSE->id";

    //$bib_entry = get_record("bibliography_entries", "citekey", $citekey);
    if ($bib_entry = get_record_sql($sql)){ 
        if (!get_record("bibliography", "course", "$COURSE->id","id", "$bib_entry->bibliographyid")){
            $bib_entry = FALSE; // se no existe entrada para a citekey
        }
    }
    if ($bib_entry) {

        //Get list of authors
        $sql = "SELECT ba.id, ba.name, ba.firstname, ba.lastname
            FROM {$CFG->prefix}bibliography_author AS ba, {$CFG->prefix}bibliography_entries_author AS bea, {$CFG->prefix}bibliography_entries AS be
            WHERE bea.entryid=be.id AND ba.id=bea.authorid AND be.id=$bib_entry->id";

        $entryauthors = get_records_sql($sql);
        $style = "author-date";
        //$style = biblio_get_style();
        return citeformat($style, $bib_entry, $entryauthors);
    } else {
        return "not found";
    }
}

/**
* Formats the bibliography footer
* Returns the printing format according to the type of entry
*
* @param string $entrytypename the type of bibliography entry
* @param string $style the general citation style (author-date in version1)
* @param object $bib_entry the bib entry object
* @param array $entryauthors an array of objects of the authors of bibentry 
*/
function citeformat($style, $bib_entry, $entryauthors){
    
    $year = $bib_entry->year;
    $last_update_date = $bib_entry->last_update_date;
    $title = $bib_entry->title;
    $context_title = $bib_entry->context_title;
    $edition = $bib_entry->edition;
    $editor = $bib_entry->editor;
    $location = $bib_entry->location;
    $publisher = $bib_entry->publisher;
    $volume = $bib_entry->volume;
    $issue = $bib_entry->issue;
    $pages = $bib_entry->pages;
    $doi = $bib_entry->doi;
    $access_date = $bib_entry->access_date;
    $url = $bib_entry->url;

    $printformat = "";
    $i=0;

    foreach ($entryauthors as $id => $author) {
        if ($i!=0) $printformat.= "and ";
        $printformat.= "$author->name ";
        $i++;
    }
    if ($year){
        $printformat.= " ($year";
        /*if ($last_update_date){
            $printformat.= ", ".date("dMY", $last_update_date);
        }*/
        $printformat.= ") ";
    }
    if ($title){
        $printformat.= "<strong>$title</strong>. ";
    }
    if ($context_title){
        $printformat.= "$context_title. ";
    }
    if ($edition){
        $printformat.= "$edition. ";
    }
    if ($editor){
        $printformat.= "$editor. ";
    }
    if ($location){
        $printformat.= "$location. ";
    }
    if ($publisher){
        $printformat.= "$publisher. ";
    }
    if ($volume){
        $printformat.= "$volume ";
        if ($issue){
            $printformat.= "($issue) ";
        }
        if ($pages){
            $printformat.= ":$pages ";
        }
    }
    if ($doi){
        $printformat.= "DOI:$doi ";
    }
    if ($access_date){
        $printformat.= date("dMY", $access_date);
    }
    if ($url){
        $printformat.= "<a href=\"$url\" target=\"_blank\">$url</a>";
    }
    return $printformat;
}

function trim_brackets($string){
    return ltrim (rtrim($string,"]"),"[");
}

////////////////////////////
//drupal filter function
function filter_xss($string, $allowed_tags = array('a', 'em', 'strong', 'cite', 'code', 'ul', 'ol', 'li', 'dl', 'dt', 'dd')) {
    // Only operate on valid UTF-8 strings. This is necessary to prevent cross
    // site scripting issues on Internet Explorer 6.

    // Store the input format
    _filter_xss_split($allowed_tags, TRUE);
    // Remove NUL characters (ignored by some browsers)
    $string = str_replace(chr(0), '', $string);
    // Remove Netscape 4 JS entities
    $string = preg_replace('%&\s*\{[^}]*(\}\s*;?|$)%', '', $string);

    // Defuse all HTML entities
    $string = str_replace('&', '&amp;', $string);
    // Change back only well-formed entities in our whitelist
    // Named entities
    $string = preg_replace('/&amp;([A-Za-z][A-Za-z0-9]*;)/', '&\1', $string);
    // Decimal numeric entities
    $string = preg_replace('/&amp;#([0-9]+;)/', '&#\1', $string);
    // Hexadecimal numeric entities
    $string = preg_replace('/&amp;#[Xx]0*((?:[0-9A-Fa-f]{2})+;)/', '&#x\1', $string);

    return preg_replace_callback('%
        (
        <(?=[^a-zA-Z!/])  # a lone <
        |                 # or
        <[^>]*(>|$)       # a string that starts with a <, up until the > or the end of the string
        |                 # or
        >                 # just a >
        )%x', '_filter_xss_split', $string);
}
function _filter_xss_split($m, $store = FALSE) {
    static $allowed_html;

    if ($store) {
        $allowed_html = array_flip($m);
        return;
    }

    $string = $m[1];

    if (substr($string, 0, 1) != '<') {
        // We matched a lone ">" character
        return '&gt;';
    }
    else if (strlen($string) == 1) {
        // We matched a lone "<" character
        return '&lt;';
    }

    if (!preg_match('%^<\s*(/\s*)?([a-zA-Z0-9]+)([^>]*)>?$%', $string, $matches)) {
        // Seriously malformed
        return '';
    }

    $slash = trim($matches[1]);
    $elem = &$matches[2];
    $attrlist = &$matches[3];

    if (!isset($allowed_html[strtolower($elem)])) {
        // Disallowed HTML element
        return '';
    }

    if ($slash != '') {
        return "</$elem>";
    }

    // Is there a closing XHTML slash at the end of the attributes?
    // In PHP 5.1.0+ we could count the changes, currently we need a separate match
    $xhtml_slash = preg_match('%\s?/\s*$%', $attrlist) ? ' /' : '';
    $attrlist = preg_replace('%(\s?)/\s*$%', '\1', $attrlist);

    // Clean up attributes
    $attr2 = implode(' ', _filter_xss_attributes($attrlist));
    $attr2 = preg_replace('/[<>]/', '', $attr2);
    $attr2 = strlen($attr2) ? ' '. $attr2 : '';

    return "<$elem$attr2$xhtml_slash>";
}

?>
