<?php
/*
BibTex Parser

Based on the WordPress plugin by Tarek Chaaban
http://www.chaaban.info/wordpress-plugin/bibtex/

Changed by Nuno Barbosa

*/
class BibTeX
{
	var $count;
	var $items;
	var $types;
	var $filename;

	function BibTeX($file = '') {
		$this->items = array(
			'citekey' => array(),
			'note' => array(),
			'abstract' => array(),
			'year' => array(),
			'group' => array(),
			'publisher' => array(),
			'page-start' => array(),
			'page-end' => array(),
			'pages' => array(),
			'address' => array(),
			'url' => array(),
			'volume' => array(),
			'chapter' => array(),
			'journal' => array(),
			'author' => array(),
			'raw' => array(),
			'title' => array(),
			'booktitle' => array(),
			'folder' => array(),
			'type' => array(),
			'linebegin' => array(),
			'lineend' => array());

		$this->filename = $file;
                $this->count = 0;
	}


function parse($file = '') {
	if($file != '') $this->setFile($file); // try to load file
	if($this->filename == '') return false; // No file loaded
	$value = array();
	$var = array();

	$this->count=-1;
	$lineindex = 0;

	$fieldcount = -1;
	$lines = file($this->filename['tmp_name']);

	if (!$lines) return;

	foreach($lines as $line) {
		
		// Added NB
		if($line[0] == "%" || $line[0] == '\n') continue; // ignores comments and empty lines
		
		$lineindex++;

		$this->items['lineend'][$this->count] = $lineindex;

		$line = trim($line);

		$raw_line = $line + '\n';

		$line=str_replace("'","`",$line);
		$seg=str_replace("\"","`",$line);

		$ps=strpos($seg,'=');

		$segtest=strtolower($seg);

		// some funny comment string
		if (strpos($segtest,'@string')!==false) {continue;}

		// pybliographer comments
		if (strpos($segtest,'@comment')!==false) {continue;}

		// normal TeX style comment
		if (strpos($seg,'%%')!==false) {continue;}

		/* ok when there is nothing to see, skip it! */
		if (!strlen($seg)) continue;

		if ("@" == $seg[0]) {

                    $this->count++;
                    $this->items['raw'][$this->count] = $line . "\r\n";

                    $ps=strpos($seg,'@');
                    $pe=strpos($seg,'{');

                    $this->types[$this->count]=trim(substr($seg, 1,$pe-1));

                    $fieldcount=-1;

                    $this->items['linebegin'][$this->count] = $lineindex;

                    $this->items['citekey'][$this->count] = trim(substr($seg, $pe+1,count($seg) - 2));

                } // #of item increase
                elseif ($ps!==false ) { // one field begins

                    $this->items['raw'][$this->count] .= $line . "\r\n";

                    $ps=strpos($seg,'=');

                    $fieldcount++;

                    $var[$fieldcount]=strtolower(trim(substr($seg,0,$ps)));

                    if ($var[$fieldcount]=='pages') {
                            $ps = strpos($seg,'=');
                            $pm = strpos($seg,'--');
                            $pe = strpos($seg,'},');

                            $pagefrom[$this->count] = substr($seg,$ps,$pm-$ps);
                            $pageto[$this->count]   = substr($seg,$pm,$pe-$pm);

                            $bp=str_replace('=','',$pagefrom[$this->count]); $bp=str_replace('{','',$bp);$bp=str_replace('}','',$bp);$bp=trim(str_replace('-','',$bp));

                            $ep=str_replace('=','',$pageto[$this->count]); $bp=str_replace('{','',$bp);$bp=str_replace('}','',$bp);;$ep=trim(str_replace('-','',$ep));
                    }

                    $pe=strpos($seg,'},');

                    if ($pe===false) {
                                    $value[$fieldcount]=strstr($seg,'=');
                    } else {
                            $value[$fieldcount]=substr($seg,$ps,$pe);
                    }

                } else {
                    if(!isset($this->items['raw'][$this->count]))
                        $this->items['raw'][$this->count] = '';
                    $this->items['raw'][$this->count] .= $line . "\r\n";
                    $pe=strpos($seg,'},');

                    if ($fieldcount > -1) {
                            if ($pe===false) {
                                    $value[$fieldcount].=' '.strstr($seg,' ');
                            } else {
                                    $value[$fieldcount] .=' '.substr($seg,$ps,$pe);
                            }
                    }
                }


            if ($fieldcount > -1) {
                    $v = $value[$fieldcount];
                    
                    $v=str_replace('`',' ',$v);
                    $v=str_replace('\"',' ',$v);
                    $v=stripslashes($v);
                    $v=str_replace('=','',$v);
                    $v=str_replace('\{','',$v);
                    $v=str_replace(' {','',$v);
                    $v=str_replace('{','',$v);
                    $v=str_replace('},','',$v);
                    $v=str_replace('}','',$v);
                    if($var[$fieldcount] != 'author') {
                        $v=str_replace(',',' ',$v);
                        $v=str_replace('\'',' ',$v);
                    }


                    $v=trim($v);

                    $this->items["$var[$fieldcount]"][$this->count]="$v";
                    /*if($var[$fieldcount] == 'year')
                        echo($this->items["$var[$fieldcount]"][$this->count] . "<br>");*/
            }

        }
	$this->normalizeAuthors();
        
    } // parse

    function normalizeAuthors() {
        $i = 0;
        while($i < $this->count) {
            if(!isset($this->items['author'][$i]) || $this->items['author'][$i] == '') {
                    $i++;
                    continue;
            }
            if($this->items['author'][$i][count($this->items['author'][$i])-1] == ',')
                $this->items['author'][$i][count($this->items['author'][$i])-1] = '';
            $i++;
        }
    }

    function setFile($file) { $this->filename = $file;}
}

?>