<?php  // $Id: lib.php,v 1.0.0.0 2010/02/01 14:00:00 nbarbosa, sleitao

define("BIBLIOGRAPHY_SHOW_ALL_INSTANCES", 0);
define("BIBLIOGRAPHY_SHOW_NOT_INSTANCIATED", -1);
define("BIBLIOGRAPHY_NO_VIEW", -1);
define("BIBLIOGRAPHY_STANDARD_VIEW", 0);
define("BIBLIOGRAPHY_DATE_VIEW", 1);
define("BIBLIOGRAPHY_AUTHOR_VIEW", 2);
define("BIBLIOGRAPHY_INSTANCE_VIEW", 3);
define("BIBLIOGRAPHY_ADDENTRY_VIEW", 4);
define("BIBLIOGRAPHY_IMPORT_VIEW", 5);
define("BIBLIOGRAPHY_EXPORT_VIEW", 6);
define("BIBLIOGRAPHY_APPROVAL_VIEW", 7);
define("BIBLIOGRAPHY_EDIT_VIEW", 8);

require_once($CFG->libdir . '/filelib.php');
require_once('locallib.php');

/**
 * Given an object containing all the necessary data,
 * (defined by the form in mod_form.php) this function
 * will create a new instance and return the id number
 * of the new instance.
 *
 * @param object $bibliography An object from the form in mod_form.php
 * @return int The id of the newly inserted bibliography record
 */
function bibliography_add_instance($bibliography) {
    global $cm;
    
    $bibliography->timecreated  = time();
    $bibliography->timemodified = $bibliography->timecreated;
    $bibliography->shortname    = generate_shortname($bibliography->name);
    
    if(count_records('bibliography','course',$bibliography->course,'shortname',$bibliography->shortname) == 0)
        return insert_record('bibliography', $bibliography);
    return 0;
}


/**
 * Given an object containing all the necessary data,
 * (defined by the form in mod_form.php) this function
 * will update an existing instance with new data.
 *
 * @param object $bibliography An object from the form in mod_form.php
 * @return boolean Success/Fail
 */
function bibliography_update_instance($bibliography) {

    $bibliography->timemodified = time();
    $bibliography->id           = $bibliography->instance;
    
    // The shortname doesn't change
    $s                       = get_record('bibliography','id', $bibliography->id);
    $bibliography->shortname = $s->shortname;
    return update_record('bibliography', $bibliography);
}


/**
 * Given an ID of an instance of this module,
 * this function will permanently delete the instance
 * and any data that depends on it.
 *
 * @param int $id Id of the module instance
 * @return boolean Success/Failure
 */
function bibliography_delete_instance($id) {

    if (! $bibliography = get_record('bibliography', 'id', $id)) {
        return false;
    }

    $result = true;

    // Delete instances
    // First, we need to get all instances to delete all instance_entries
    $instances = get_records('bibliography_instances', 'bibliographyid', $bibliography->id);
    if (! delete_records('bibliography_instances', 'bibliographyid', $bibliography->id)) {
        $result = false;
    }

    // Next, we delete all instance_entries
    if($instances) {
        foreach($instances as $instance) {
            if (! delete_records('bibliography_inst_entries', 'instanceid', $instance->id)) {
                if(! $result) $result = false;
            }
        }
    }

    // Get entries to delete comments
    $entries = get_records('bibliography_entries','bibliographyid',$bibliography->id);
    if($entries)
        foreach($entries as $entry) {
            if(! delete_records('bibliography_comments','entryid',$entry->id)) {
                if(! $result) $result = false;
            }
        }
    
    // Then, delete entries
    if(! delete_records('bibliography_entries','bibliographyid',$bibliography->id)) {
        if(! $result) $result = false;
    }

    if (! delete_records('bibliography', 'id', $bibliography->id)) {
        if(! $result) $result = false;
    }

    return $result;
}


/**
 * Return a small object with summary information about what a
 * user has done with a given particular instance of this module
 * Used for user activity reports.
 * $return->time = the time they did it
 * $return->info = a short text description
 *
 * @return null
 * @todo Finish documenting this function
 */
function bibliography_user_outline($course, $user, $mod, $bibliography) {
    return $return;
}


/**
 * Print a detailed representation of what a user has done with
 * a given particular instance of this module, for user activity reports.
 *
 * @return boolean
 * @todo Finish documenting this function
 */
function bibliography_user_complete($course, $user, $mod, $bibliography) {
    return true;
}


/**
 * Given a course and a time, this module should find recent activity
 * that has occurred in bibliography activities and print it out.
 * Return true if there was output, or false is there was none.
 *
 * @return boolean
 * @todo Finish documenting this function
 */
function bibliography_print_recent_activity($course, $isteacher, $timestart) {
    return false;  //  True if anything was printed, otherwise false
}


/**
 * Function to be run periodically according to the moodle cron
 * This function searches for things that need to be done, such
 * as sending out mail, toggling flags etc ...
 *
 * @return boolean
 * @todo Finish documenting this function
 **/
function bibliography_cron () {
    return true;
}


/**
 * Must return an array of user records (all data) who are participants
 * for a given instance of bibliography. Must include every user involved
 * in the instance, independient of his role (student, teacher, admin...)
 * See other modules as example.
 *
 * @param int $bibliographyid ID of an instance of this module
 * @return mixed boolean/array of students
 */
function bibliography_get_participants($bibliographyid) {
    return false;
}


/**
 * This function returns if a scale is being used by one bibliography
 * if it has support for grading and scales. Commented code should be
 * modified if necessary. See forum, BIBLIOGRAPHY or journal modules
 * as reference.
 *
 * @param int $bibliographyid ID of an instance of this module
 * @return mixed
 * @todo Finish documenting this function
 */
function bibliography_scale_used($bibliographyid, $scaleid) {
    return false;
}


/**
 * Checks if scale is being used by any instance of bibliography.
 * This function was added in 1.9
 *
 * This is used to find out if scale used anywhere
 * @param $scaleid int
 * @return boolean True if the scale is used by any bibliography
 */
function bibliography_scale_used_anywhere($scaleid) {
    return false;
}


/**
 * Execute post-install custom actions for the module
 * This function was added in 1.9
 *
 * @return boolean true if success, false on error
 */
function bibliography_install() {
    global $CFG;

    require_once('install.php');
    
    for($i = 0; $i < count($insert_sql); $i++) {
        echo $insert_sql[$i];
        if(!execute_sql($insert_sql[$i])) return false;
    }

    return true;
}


/**
 * Execute post-uninstall custom actions for the module
 * This function was added in 1.9
 *
 * @return boolean true if success, false on error
 */
function bibliography_uninstall() {
    return true;
}

///
/// Other Functions
///

//---------------------
/// NB Functions
//---------------------
/**
 * Returns the instances of a bibliography
 *
 * @param int $id which is the id of the instance
 * @return an Object containing the results, or false
 */
function bibliography_get_instances( $id, $hook=0) {
    global $CFG;
    if($hook > 0) {
        $query = 'select * from ' . $CFG->prefix . 'bibliography_instances '
               . 'where id = ' . $hook . ' and bibliographyid = ' . $id;
        return get_records_sql($query);
    }
    return get_records("bibliography_instances","bibliographyid",$id);
}
/**
 * Returns the entries without instance
 *
 * @param int $id which is the id of the instance
 * @return an Object containing the results, or false
 */
function bibliography_get_noinstances($id) {
    global $CFG;
    // Instanciated entries
    $query = 'SELECT be.* FROM ('. $CFG->prefix . 'bibliography_entries as be join '
            . $CFG->prefix . 'bibliography_inst_entries as bie on be.id = bie.entryid) '
            . 'where be.bibliographyid = ' . $id;
    $entriesInstances = get_records_sql($query);

    // Not instanciated
    $finalquery = 'SELECT * FROM '
         . $CFG->prefix . 'bibliography_entries '
         . 'where bibliographyid = ' . $id;
    if($entriesInstances)
        foreach($entriesInstances as $entry) {
            $finalquery .= ' and id <> ' . $entry->id;
        }
    return get_records_sql($finalquery);
}

/**
 * Returns all entries info belonging to an instancs
 *
 * @param int $id which is the id of the instance
 * @param int $entrytype (optional) if you need to get a explicit type
 * @return Object with all records or false
 */
function bibliography_get_instance_entries( $id, $entrytype=0) {
    global $CFG;
    $query = 'select be.* from '
            . '(' . $CFG->prefix . 'bibliography_inst_entries as bie join '
            . $CFG->prefix .'bibliography_entries as be on bie.entryid = be.id) '
            . 'where bie.instanceid = ' . $id;
    if($entrytype) $query .= ' and be.entrytypeid = ' . $entrytype;
    return get_records_sql($query);
}

/**
 * Returns all entries info belonging to an instance of a module
 *
 * @param int $id which is the id of the instance
 * @param int $entrytype (optional) if you need to get a explicit type
 * @return Object with all records or false
 */
function bibliography_get_module_entries($id,$entrytype=0) {
    global $CFG;
    $query = 'select * from '
            . $CFG->prefix .'bibliography_entries '
            . 'where bibliographyid = ' . $id;
    if($entrytype) $query .= ' and entrytypeid = ' . $entrytype;
    return get_records_sql($query);
}

/**
 * Gets the authors of an entry
 *
 * @param int $id is the id of the entry
 * @return Object or false
 */
function bibliography_get_entry_authors($id) {
    global $CFG;
    $query = 'select author.* from '
           . '(' . $CFG->prefix . 'bibliography_entries_author as bea join '
           . $CFG->prefix . 'bibliography_author as author on bea.authorid = author.id) '
           . 'where bea.entryid = ' . $id . ' order by author.lastname';
    return get_records_sql($query);
}

/**
 * Gets the name of the type of an entry
 *
 * @param int $id of the entry
 * @return string with the name of the type, false if not succesfull
 */
function bibliography_get_entry_type($id) {
    $entry = get_record("bibliography_entries",'id',$id);
    $type  = get_record("bibliography_entrytype",'id',$entry->entrytypeid);
    return $type->typename;
}

/**
 * Gets the citation key of an entry in an instance
 *
 * @param int $instanceid
 * @param int $entryid
 * @return int representing the citekey or false if no record found
 */
function bibliography_get_citekey($instanceid,$entryid) {
    $record = get_record("bibliography_inst_entries","instanceid",$instanceid,"entryid",$entryid);
    return $record->citekey;
}

/**
 * Returns the fields and the field name used in the entry type
 *
 * @param int $id is the entry type id
 * @return Object with the result of the query, false if query unsuccesfull
 */
function bibliography_get_entrytype_fields($id) {
    global $CFG;
    $query = 'select fieldname, fieldtypename from ('
           . $CFG->prefix .'bibliography_entrytype_field as bef join '
           . $CFG->prefix .'bibliography_field_name as bfn on bfn.id = bef.fieldnameid)'
           . ' where bef.entrytypeid = ' . $id;
    return get_records_sql($query);
}

/**
 * Gets the value of an field in an entry
 *
 * @param int $id is the id of the entry
 * @param string $field is the field to access
 * @return string if success and false if not
 */
function bibliography_get_field_content($id,$field) {
    $record = get_record("bibliography_entries","id",$id);
    return $record->$field;
}

/**
 * Gets all years in an instance
 *
 * @param int $id is the id of the instance
 * @return Object containing all years
 **/
function bibliography_years_in_instance($id) {
    global $CFG;
    $query = 'select distinct year from '
            . $CFG->prefix . 'bibliography_inst_entries as bie join '
            . $CFG->prefix . 'bibliography_entries as be '
            . 'on bie.entryid=be.id '
            . 'where bie.instanceid = ' . $id;
    return get_records_sql($query);
}

/**
 * Gets all entries of an instance ordered by year
 *
 * @param int $id id of the instance
 * @return Object with data or false if no result
 */
function bibliography_get_entriesdata_by_year($id, $page=1, $pages=1, $perpage=20) {
    global $CFG;

    $asc = '';
    if($pages > 1) $asc = ' ASC LIMIT ' . ($perpage * ($page - 1)) . ', ' . $perpage;

    $sql = 'SELECT * FROM '
         . $CFG->prefix . 'bibliography_entries '
         . 'where bibliographyid = ' . $id . ' order by year, citekey' . $asc;
    return get_records_sql($sql);
}

/**
 * Gets all entries of an instance with and ordered by citation key
 *
 * @param int $id id of the instance
 * @return Object with data or false if no result
 */
function bibliography_get_entries_standard($id, $entryid=0, $page=1, $pages=1, $perpage=20) {
    global $CFG;

    $asc = '';
    if($pages > 1) $asc = ' ASC LIMIT ' . ($perpage * ($page - 1)) . ', ' . $perpage;

    $e = '';
    if($entryid) $e = ' and id = ' . $entryid;

    /*
    $sql = 'SELECT *, substring(citekey,' . (strlen($instance->shortname) + 2) .') as c, '
         . 'lpad(substring(citekey,' . (strlen($instance->shortname) + 2) .'),8,"0") as number FROM '
         . $CFG->prefix . 'bibliography_entries '
         . 'where bibliographyid = ' . $id . $e . ' group by c order by number' . $asc;
     */

    $sql = 'SELECT * FROM '
         . $CFG->prefix . 'bibliography_entries '
         . 'where bibliographyid = ' . $id . $e . ' order by citekey' . $asc;
    return get_records_sql($sql);
}

/**
 * This function returns an array of bibliography entrytype names
 *
 * @return array
 */
function bibliography_get_bibliography_entrytypes(){
	global $CFG;
	$sql = "SELECT id, typename 
		FROM {$CFG->prefix}bibliography_entrytype ORDER BY id";
	$entrytypes = get_records_sql($sql);
	$entrytypes_array = array();
	foreach ($entrytypes as $id => $entrytype) {
   		array_push ($entrytypes_array, $entrytype->typename);
	}
	return $entrytypes_array;
}

/**
 * Creates a table given the module instance and the author
 *
 * @param int $author id number
 * @param int $instance id number
 **/
function bibliography_build_table_by_author($author, $instance, $letter=0) {
    global $CFG;

    
    $query = 'select be.* from '
           . '((' . $CFG->prefix . 'bibliography_entries_author as bea join '
           . $CFG->prefix . 'bibliography_author as author on bea.authorid = author.id) join '
           . $CFG->prefix . 'bibliography_entries as be on bea.entryid = be.id)'
           . 'where be.bibliographyid = ' . $instance . ' and author.id = ' . $author
           . ' order by author.lastname';
    $results = get_records_sql($query);

    // Author information
    $authInfo = get_record("bibliography_author","id",$author);

    if($letter)
        if(!preg_match('/^' . $letter . '/',$authInfo->lastname)) return false;

    $table = array('head'  => array($authInfo->name),
                   'align' => array("center"),
                   'data'  => array());
    $index = 0;
    foreach($results as $result) {
        $table['data'][$index]   = array();
        $table['data'][$index][] = format_entry($result,array("authors"));
        $index++;
    }
    print_table(parseArrayToObject($table));
    echo "<br />";
    return true;
}

/**
 * Gets all associated authors in an instance
 *
 * @param int $id of the author
 * @return result from the query
 **/
function bibliography_get_all_authors($id, $page=1, $pages=1, $perpage=20,$letter='') {
    global $CFG;

    $l = '';
    if($letter != '') $l = ' and lastname like "' . $letter . '%" ';

    $asc = '';
    if($pages > 1) $asc = ' ASC LIMIT ' . ($perpage * ($page - 1)) . ', ' . $perpage;

    $query = 'select distinct author.* from '
           . '((' . $CFG->prefix . 'bibliography_entries_author as bea join '
           . $CFG->prefix . 'bibliography_author as author on bea.authorid = author.id) join '
           . $CFG->prefix . 'bibliography_entries as be on bea.entryid = be.id)'
           . 'where be.bibliographyid = ' . $id . $l . ' order by lastname' . $asc;
    return get_records_sql($query);
}

/**
 * Prints or gets the icons do delete or edit an entry (hack from the bibliography module)
 *
 * @param int $course
 * @param Object $cm
 * @param Object $bibliography
 * @param Object $entry
 * @param string $mode
 * @param string $hook
 * @param string $type
 * @return string
 **/
function bibliography_print_entry_icons($course, $cm, $bibliography, $entry, $mode='',$hook='', $type = 'print') {
    global $USER, $CFG, $mode;

    $context = get_context_instance(CONTEXT_MODULE, $cm->id);
    $output = false;

    $return = '<span class="commands"><div style="text-align:right;">';

    $return .= bibliography_print_entry_commentslink($course, $cm, $bibliography, $entry, $mode, $hook,'html');

    if (has_capability('mod/bibliography:comment', $context) and $bibliography->allowcomments) {
        $output = true;
        $return .= ' <a title="' . get_string('addcomment','bibliography') . '" href="comment.php?action=add&amp;eid='.$entry->id.'"><img src="comment.gif" class="iconsmall" alt="'.get_string('addcomment','bibliography').'" /></a>';
    }

    if (has_capability('mod/bibliography:manageentries', $context) || bibliography_students_allowed($cm->instance, $USER->id, $entry->id)) {
        
        $icon = "$CFG->pixpath/t/delete.gif";

        $output = true;
        $return .= " <a title=\"" . get_string("delete") . "\" href=\"deleteentry.php?id=$cm->id&amp;mode=delete&amp;entry=$entry->id&amp;prevmode=$mode&amp;hook=$hook\"><img src=\"";
        $return .= $icon;
        $return .= "\" class=\"iconsmall\" alt=\"" . get_string("delete") ."\" /></a> ";

        $return .= " <a title=\"" . get_string("edit") . "\" href=\"edit.php?id=$cm->id&amp;e=$entry->id&amp;mode=$mode&amp;hook=$hook\"><img src=\"$CFG->pixpath/t/edit.gif\" class=\"iconsmall\" alt=\"" . get_string("edit"). "\" /></a>";

    } 
    $return .= "&nbsp;&nbsp;"; // just to make up a little the output in Mozilla ;)

    // Puts the name of the user that added the entry on the left
    $user   = get_record("user","id",$entry->userid);
    if($user)
        $return .= '<br /><div style="font-size: x-small;">' . get_string('addedby','bibliography')
            . ' ' . $user->firstname . ' ' . $user->lastname . '</div>';
    // ----------------------------------------------------------

    $return .= '</div></span>';

    //If we haven't calculated any REAL thing, delete result ($return)
    if (!$output) {
        $return = '';
    }
    //Print or get
    if ($type == 'print') {
        echo $return;
    } else {
        return $return;
    }
}

/**
 * Prints a comment link
 * @param Object $course
 * @param Object $cm
 * @param Object $bibliography
 * @param Object $entry
 * @param int $mode
 * @param string $hook
 * @param string $type
 * @return string
 **/
function bibliography_print_entry_commentslink($course, $cm, $bibliography, $entry,$mode,$hook, $type = 'print') {
    $return = '';

    $count = count_records('bibliography_comments','entryid',$entry->id);
    if ($count) {
        $return = '';
        $return .= "<a href=\"comments.php?id=$cm->id&amp;eid=$entry->id\">$count ";
        if ($count == 1) {
            $return .= get_string('comment', 'bibliography');
        } else {
            $return .= get_string('comments', 'bibliography');
        }
        $return .= '</a>';
    }

    if ($type == 'print') {
        echo $return;
    } else {
        return $return;
    }
}

/**
 * Gets an entry by id
 * @param int $id
 * @return Object
 **/
function bibliography_get_entry($id) {
    return get_record("bibliography_entries","id",$id);
}

/**
 * Prints a comment on a table
 * @param int $id
 * @param bool $single
 **/
function bibliography_print_commenton_table($id, $single=false) {
    $data = bibliography_get_entry($id);
    
    // Just to know if we print "CommentS on" or "Comment on"
    if($single)
        $commentson = get_string('commenton','bibliography');
    else
        $commentson = get_string('commentson','bibliography');
        
    // Table
    $table = array("head"  => array($commentson),
                   "align" => array("center"),
                   "data"  => array(array(format_entry($data,array("no_lower_icons")))));
    print_table(parseArrayToObject($table));
}

/**
 * Print a comment
 * @global Object $CFG
 * @global Object $USER
 * @param Object $course
 * @param Object $cm
 * @param Object $bibliography
 * @param Object $entry
 * @param Object $comment
 * @param bool $noicons
 **/
function bibliography_print_comment($course, $cm, $bibliography, $entry, $comment, $noicons=false) {
    global $CFG, $USER;

    $context = get_context_instance(CONTEXT_MODULE, $cm->id);

    $user = get_record('user', 'id', $comment->userid);
    $strby = get_string('writtenby','bibliography');
    $fullname = fullname($user, has_capability('moodle/site:viewfullnames', get_context_instance(CONTEXT_COURSE, $course->id)));

    echo '<div class="boxaligncenter">';
    echo '<table class="bibliographycomment" cellspacing="0">';
    echo '<tr valign="top">';
    echo '<td class="left picture">';
    print_user_picture($user, $course->id, $user->picture);
    echo '</td>';
    echo '<td class="entryheader">';

    $fullname = fullname($user, has_capability('moodle/site:viewfullnames', get_context_instance(CONTEXT_COURSE, $course->id)));
    $by = new object();
    $by->name = '<a href="'.$CFG->wwwroot.'/user/view.php?id='.$user->id.'&amp;course='.$course->id.'">'.$fullname.'</a>';
    $by->date = userdate($comment->timemodified);
    echo '<span class="author">'.get_string('bynameondate', 'forum', $by).'</span>';

    echo '</td></tr>';

    echo '<tr valign="top"><td class="left side">';
    echo '&nbsp;';
    echo '</td><td class="entry">';

    $options = new object();
    $options->trusttext = true;
    echo format_text($comment->entrycomment);

    echo '<div class="icons commands">';

    $ineditperiod = ((time() - $comment->timemodified <  $CFG->maxeditingtime) || $bibliography->editalways);
    if ( (($bibliography->allowcomments &&  $ineditperiod && $USER->id == $comment->userid)  || has_capability('mod/bibliography:managecomments', $context)) && !$noicons) {
        echo "<a href=\"comment.php?cid=$comment->id&amp;action=edit\"><img
               alt=\"" . get_string("edit") . "\" src=\"$CFG->pixpath/t/edit.gif\" class=\"iconsmall\" /></a> ";
    }
    if ( (($bibliography->allowcomments && $USER->id == $comment->userid) || has_capability('mod/bibliography:managecomments', $context)) && !$noicons ) {
        echo "<a href=\"comment.php?cid=$comment->id&amp;action=delete\"><img
               alt=\"" . get_string("delete") . "\" src=\"$CFG->pixpath/t/delete.gif\" class=\"iconsmall\" /></a>";
    }

    echo '</div></td></tr>';
    echo '</table></div>';

}

/**
 * Prints the categories menu
 * @global Object $CFG global configuration var
 * @param Object $cm
 * @param Object $bibliography
 * @param string $hook
 * @param string $instance
 **/
function bibliography_print_instances_menu($cm, $bibliography, $hook, $instance) {

     global $CFG;

     $context = get_context_instance(CONTEXT_MODULE, $cm->id);

     echo '<table border="0" width="100%">';
     echo '<tr>';

     echo '<td align="center" style="width:20%">';
     if (has_capability('mod/bibliography:manageinstances', $context)) {
             $options['id'] = $cm->id;
             $options['mode'] = 'cat';
             $options['hook'] = $hook;
             echo print_single_button("editinstances.php", $options, get_string("editinstances","bibliography"), "get");
     }
     echo '</td>';

     echo '<td align="center" style="width:60%">';
     echo '<b>';

     $menu[BIBLIOGRAPHY_SHOW_ALL_INSTANCES] = get_string("allinstances","bibliography");
     $menu[BIBLIOGRAPHY_SHOW_NOT_INSTANCIATED] = get_string("notinstanciated","bibliography");

     $instances = get_records("bibliography_instances", "bibliographyid", $bibliography->id, "name ASC");
     $selected = '';
     if ( $instances ) {
          foreach ($instances as $currentinstance) {
                 $url = $currentinstance->id;
                 if ( $instance ) {
                     if ($currentinstance->id == $instance->id) {
                         $selected = $url;
                     }
                 }
                 $menu[$url] = clean_text($currentinstance->name); //Only clean, not filters
          }
     }
     if ( !$selected ) {
         $selected = BIBLIOGRAPHY_SHOW_NOT_INSTANCIATED;
     }

     if ( $instance ) {
        echo format_text($instance->name, FORMAT_PLAIN);
     } else {
        if ( $hook == BIBLIOGRAPHY_SHOW_NOT_INSTANCIATED ) {

            echo get_string("entrieswithoutinstance","bibliography");
            $selected = BIBLIOGRAPHY_SHOW_NOT_INSTANCIATED;

        } elseif ( $hook == BIBLIOGRAPHY_SHOW_ALL_INSTANCES ) {

            echo get_string("allinstances","bibliography");
            $selected = BIBLIOGRAPHY_SHOW_ALL_INSTANCES;

        } else $selected = $hook;
     }
     echo '</b></td>';
     echo '<td align="center" style="width:20%">';

     echo popup_form("$CFG->wwwroot/mod/bibliography/view.php?id=$cm->id&amp;mode=instance&amp;hook=", $menu, "catmenu", $selected, "",
                      "", "", false);

     echo '</td>';
     echo '</tr>';

     echo '</table>';
}

/**
 * Returns the file type given the typebibtexname
 * @param string $typebibtexname the bibtex name
 * @return Object
 **/
function bibliography_get_fields_typebibtexname($typebibtexname) {
    if($typebibtexname != '')
        $bibtex = get_record('bibliography_entrytype','typebibtexname',$typebibtexname);
    else $bibtex = false;
    // In case the type doesn't exist, get MISC
    if(!$bibtex)  $bibtex = get_record('bibliography_entrytype','typebibtexname','MISC');
    $fields = bibliography_get_entrytype_fields($bibtex->id);
    return $fields;
}

function bibliography_generate_export_file($bibliography, $hook = "") {
    $content = "";
    $barn = chr(10);
    $entries = bibliography_get_entries_standard($bibliography->id);
    
    foreach($entries as $entry) {
        $entrytype = get_record("bibliography_entrytype","id",$entry->entrytypeid);

        // Add bibtex type name
        if(isset($entrytype->typebibtexname) && strlen($entrytype->typebibtexname) > 0)
            $bibtexname = $entrytype->typebibtexname;
        else $bibtexname = 'MISC';
        $content .= '@' . $bibtexname . '{';

        // Add citation key
        $content .= $entry->citekey . ',' .$barn;

        // Authors
        $authors = bibliography_get_entry_authors($entry->id);
        $a       = 'author = {';
        $and     = false; // do we add an "and"?
        foreach($authors as $author) {
            if($and) $a .= ' and '; // its only false at the beginning
            $a  .= $author->name;
            $and = true;
        }
        $a .= '}';
        $content .= $a;

        // Other fields
        $fields = bibliography_get_fields_typebibtexname($bibtexname);
        foreach($fields as $field) {
            $fieldname = $field->fieldname;
            if(!isset($entry->$fieldname)) continue;
            if($fieldname != 'author' && $fieldname != 'citekey' && $entry->$fieldname != '') {
                $f = '';
                if(!isset($field->fieldtypebibtexname)) $f = $field->fieldtypename;
                else $f = $field->fieldtypebibtexname;
                $content .= ',' .$barn . $f . ' = {' . $entry->$fieldname . '}';
            }
        }

        $content .= $barn . '}' . $barn;
    }
    return $content;
}

/**
 * Returns the page data needed
 * @global Object $CFG
 * @param int $id module id
 * @param int $mode view mode
 * @return Object
 */
function bibliography_get_pages($id, $mode, $letter) {
    global $CFG;
    $result = new stdClass();
    $record = get_record("bibliography","id",$id);
    $result->perpage = $record->entbypage;
    switch($mode) {
        case BIBLIOGRAPHY_STANDARD_VIEW: // By entry (one table by page
        case BIBLIOGRAPHY_DATE_VIEW: // By year table
            $result->pages   = ceil(count_records("bibliography_entries","bibliographyid",$id) / $result->perpage);
            break;
        case BIBLIOGRAPHY_AUTHOR_VIEW: // By author table
            if($letter == '') {
                $nAuthors = ceil(count(bibliography_get_all_authors($id)) / $result->perpage);
                $result->pages = $nAuthors;
            } else $result->pages = 1;
            break;
        case BIBLIOGRAPHY_INSTANCE_VIEW: // By instance table
            if(!isset($hook)) $hook = 0;
            if($hook == 0) {
                $result->pages = ceil(count_records('bibliography_instances','bibliographyid',$id) / $result->perpage);
            } else $result->pages = 1;
            break;
        default: break;
    }
    return $result;
}


function bibliography_check_keyword($id,$search) {
    global $CFG;
    $query = 'SELECT count(*) FROM '. $CFG->prefix . 'bibliography_entries_keyword as bek join '
            . $CFG->prefix . 'bibliography_keywords as bk on bk.id = bek.keywordid where bk.keyword '
            . 'LIKE "%' . $search . '%" and bek.entryid = ' . $id;
    return count_records_sql($query);
}

function bibliography_students_allowed($id, $userid=0, $entryid=0) {
    $record = get_record("bibliography","id",$id);
    if($userid) {
        $recorduser = get_record("bibliography_entries","bibliographyid",$id,"userid",$userid,"id",$entryid);
        return $record->allowstudents && $recorduser;
    }
    return $record->allowstudents;
}
/// SL Functions
/**
 * This function deletes records from bibliography_inst_entries for instances of a bibliography
 *
 * @return nothing
 */
function bibliography_delete_inst_entries($entryid, $bibliographyid){
    // the idea is to delete only instance entries which instance belongs to a certain bibliography
    global $CFG, $db;
    $sql = "DELETE
            FROM {$CFG->prefix}bibliography_inst_entries
            WHERE instanceid IN (SELECT bi.id
	    FROM".$CFG->prefix."bibliography_instances bi
	    WHERE bi.bibliographyid = ?)
	    AND entryid = ?";
    $params = array($bibliographyid, $entryid);
    $db->execute($sql,$params);
}

/**
 * This function finds duplicate entries in bibliography
 *
 * @return the entry or FALSE
 */
function bibliography_get_dupentries($title, $bibliographyid, $entrytypeid, $year, $context_title){
    global $CFG;
    $dupentries = get_record_sql('SELECT *
				FROM '.$CFG->prefix.'bibliography_entries
				WHERE lower(title) = "'.moodle_strtolower($title).'"
				AND bibliographyid = '.$bibliographyid.'
            			AND entrytypeid = '.$entrytypeid.'
                		AND year = "'.$year.'"
				AND context_title = "'.$context_title.'"');
    return $dupentries;
}

/**
 * This function generates a citekey
 *
 * @return string citekey
 */
function bibliography_create_citekey($bibliography, $e, $courseid=NULL, $citekey=NULL, $authors=NULL, $year=0){
    global $CFG;

    if (!$citekey || $citekey==''){

        if($authors) { // Use authors name if available
            $p = strpos($authors,','); $name = '';
            if($p === false) {
                $aux = explode(' ', str_replace(array("\r\n","\n","\r"),' ',$authors));
                if(count($aux) >= 2) $name = $aux[1];
                else $name = $aux[0];
            } else $name = substr($authors, 0, $p);
            $citekey =  $name . $year;
            // Number of record with <shortname>-<id>
            $sql = 'SELECT count(*) FROM '
                 . $CFG->prefix . 'bibliography_entries m where bibliographyid=' . $bibliography->id
                 . ' and citekey like "' . $citekey .'"';
            if(count_records_sql($sql) == 0) return $citekey;
        } else {
            // Use shortname
            $citekey = $bibliography->shortname;
        }

        // Number of record with <shortname>-<id>
        $res = get_field_sql('SELECT m.citekey as citekey FROM '
                     . $CFG->prefix . 'bibliography_entries m where bibliographyid=' . $bibliography->id
                     . ' and citekey like "' . $citekey .'-%" order by citekey DESC');
        $n = 1;
        if(!$res) return $citekey."-". str_pad($n,5,'0', STR_PAD_LEFT);

        
        $check = explode('-',$res);
        //if(count($res) == 1) return $check[0]."-". str_pad($n + 1,5,'0', STR_PAD_LEFT);

        $index = 1; $newCitekey = $check[0]; $calculate = true;
        while(isset($check[$index])) {
            if(preg_match('/[0-9]+/',$check[$index]) && $calculate) {
                $value = intval($check[$index]);
                $newCitekey .= '-' . str_pad($value + 1,5,'0', STR_PAD_LEFT);
                $calculate = false;
            } else $newCitekey .= '-' . $check[$index];
            $index++;
        }
        return $newCitekey;
    } else {
        // Verify that it doesn't exist
        $sql = "SELECT count(*) as citekey
            FROM {$CFG->prefix}bibliography_entries AS be
            JOIN {$CFG->prefix}bibliography AS b ON (be.bibliographyid = b.id)
            WHERE b.course='".$courseid."'
            AND be.citekey LIKE '".$citekey."'";
        $count = count_records_sql($sql);
        
        if($count == 1) return $citekey;

        // If it exists, add the following info
        $sql = "SELECT be.citekey as citekey
            FROM {$CFG->prefix}bibliography_entries AS be
            JOIN {$CFG->prefix}bibliography AS b ON (be.bibliographyid = b.id)
            WHERE b.course='".$courseid."'
            AND be.citekey LIKE '".$citekey."-%' order by citekey DESC";
        $newN  = get_field_sql($sql);

        if(!$newN) {
            $check = array();
            $check[] = $citekey;
            $check[] = '0';
        } else $check = explode('-', $newN);

        $value = 1;
        // There are some very rare cases that this if is needed... Mostly it isn't
        if(preg_match('/[0-9]+/',$check[count($check) - 1])) $value = intval($check[count($check) - 1]); // For increment
        
        $citekey .= "-".str_pad($value + 1,5,'0', STR_PAD_LEFT);
    }
    return $citekey;
}

function bibliography_insert_authors($authorsdataobj, $e, $explodechar) {
    delete_records("bibliography_entries_author", "entryid", $e);
    if ( isset($authorsdataobj->authors) ) {
        if ( $authors = explode($explodechar, $authorsdataobj->authors) ) {
            foreach ($authors as $author) {
                if ($author) {
                        $author = trim($author);
                        // Check format "Lastname, Othernames" or "Firstname Middlename Lastname"
                        if (!$pos = strpos($author, ",")) {
                                $author = bibliography_change_author_name_format($author);
                        }
                        if (!record_exists("bibliography_author", "name", $author)){ // if the author does not exist insert it
                                unset($newauthor);
                                $newauthor->name = $author;
                                $pos = strpos($author, ",");
                                $newauthor->firstname = bibliography_get_author_firstname($author, $pos);
                                $newauthor->lastname = bibliography_get_author_lastname($author, $pos);
                                $authorid = insert_record("bibliography_author", $newauthor, true);
                                //echo "authorid=".$authorid."<br />";
                        } else { // if the author already exists get the id to insert it in entries_author table
                                $author_obj = get_record("bibliography_author", "name", $author);
                                $authorid = $author_obj->id;
                        }
                        if (!record_exists("bibliography_entries_author", "entryid", $e, "authorid", $authorid)){ // only if the record does not already exist in entries_author table
                                unset($newauthor_entry);
                                $newauthor_entry->authorid = $authorid;
                                $newauthor_entry->entryid = $e;
                                $newauthor_entry->secondary = 0; // a resolver!!
                                if (!insert_record("bibliography_entries_author", $newauthor_entry, false)){
                                        return false;
                                }
                        }

                }
            }
        }
    }

    return true;
}
/**
 * Get the author firstname
 * @param string author name in format Lastname, Firstname Middlename, int $pos position of comma
 * @return array the author firstname
 */
function bibliography_get_author_firstname($author_name, $pos){
    $author_nospace  = str_replace(", ", ",", $author_name); // retirar espacos das virgulas
    $newpos = strpos($author_nospace, ",");
    $firstname = substr($author_nospace, $newpos, strpos($author_nospace." ", " ")-$newpos); // retira o primeiro nome apos a virgula
    $firstname = trim ($firstname,",");
    return trim($firstname);
}

/**
 * Get the author lastname
 * @param string author name in format Lastname, Firstname Middlename, int $pos position of comma
 * @return string the author lastname 
 */
function bibliography_get_author_lastname($author_name, $pos){
	$lastname = substr($author_name, 0 ,$pos);
	return trim($lastname);
}

/**
 * Change author name format to Lastname, Firstname Middlename
 * @param string author name in format Firstname Middlename Lastname
 * @return string the author name in format Lastname, Firstname Middlename
 */
function bibliography_change_author_name_format($author){
    // colocar no formato "Lastname, Othernames"
    $author_names = explode(" ", $author);
    $author_new = trim(array_pop($author_names)).",";

    foreach ($author_names as $author_name) $author_new .= " ".trim($author_name);

    $author_new = trim ($author_new,",");
    return trim($author_new);
}
?>