<?php // $Id: locallib.php,v 1.0.0.0 2010/02/01 14:00:00 nbarbosa, sleitao

///// Contains other secondary functions

/**
 * Converts an array to an object
 *
 * @param Array $array
 * @return Object
 */
function parseArrayToObject($array) {
   $object = new stdClass();
   if (is_array($array) && count($array) > 0) {
      foreach ($array as $name=>$value) {
         $name = strtolower(trim($name));
         if (!empty($name)) {
            $object->$name = $value;
         }
      }
   }
   return $object;
}

/**
 * Orders the data by author
 *
 * @param Array $data
 * @return Array
 */
function order_by_author($data) {
    $result  = array();
    $indexes = array();
    // The array is ordered if the size of $indexes is equal to the size of $data
    $lower = 0;
    while(count($indexes) != count($data)) {
        for($i = 0; $i < count($data); $i++) {
            if(!in_array($i,$indexes)) {
                if(strcmp($data[$i][0], $data[$lower][0]) <= 0) $lower = $i;
            } else 
                if($i < count($data) - 1) $lower = $i + 1;
                else $lower = 0;
        }
        $indexes[] = $lower;
        $lower     = 0;
    }
    
    // Create the result
    for($i = 0; $i < count($indexes); $i++) {
        $result[] = $data[$indexes[$i]];
    }
    return $result;
}

/**
 * Formats an entry by its type
 *
 * Note: This function may be re-done to support custom types of entries
 *
 * @param Object $data to format and convert to string
 * @param array $exclude some field
 * @return string for presenting the results
 */
function format_entry($data,$exclude=array()) {
    global $CFG, $COURSE, $cm;
    $string = "";
    // Get entry type name (will be needed for custom entries)
    // $entrytyped = get_record("bibliography_entrytype", "id", $data->entrytypeid);
    // $entrytype  = $entrytyped->typename;
    
    // Query to get needed fields
    $sql = 'select fieldname, fieldtypename from '
         . $CFG->prefix .'bibliography_entrytype_field as bef join '
         . $CFG->prefix .'bibliography_field_name as bfn on bfn.id = bef.fieldnameid '
         . 'where bef.entrytypeid = ' . $data->entrytypeid;
    $fields = get_records_sql($sql);
    // Add fields contents
    if($fields) {
        foreach($fields as $field) {
            // We will only add field if not present in exclude
            if(!in_array($field->fieldname,$exclude)) {
                $fieldname     = $field->fieldname;
                $fieldtypename = $field->fieldtypename;
                if(!isset($data->$fieldname)) continue;
                if(strlen($data->$fieldname) > 0 && $data->$fieldname) { // No print of zero size strings or zero values
                    switch($fieldname) {
                        case "year":
                           $string .= '<b>' . get_string("year","bibliography") .  ':</b> ' . $data->year . ' | ';
                           break;
                        case "title":
                            $string .= '<b>' . get_string("title","bibliography") .  ':</b> "<i>' . $data->title . '</i>"<br/>';
                            break;
                        case "last_update_date":
                            $string .= '<b>' . get_string($fieldtypename,"bibliography") . ' (mm/dd/yyyy):</b> '
                                    .  date("m/d/Y",$data->last_update_date) . ' | ';
                            break;
                        case "retrieved_update_date":
                            $string .= '<b>' . get_string($fieldtypename,"bibliography") . ' (mm/dd/yyyy):</b> '
                                    .  date("m/d/Y",$data->retrieved_update_date) . ' | ';
                            break;
                        case "url": break;
                        default:
                                $string .= '<b>' . get_string($fieldtypename,"bibliography") . ':</b> '
                                          . $data->$fieldname . ' | ';
                    }
                }
            }
        }
    }

    // Add entry type?
    $entrytype = '';
    if(!in_array("entrytype",$exclude)) {
        $result    = get_record("bibliography_entrytype",'id',$data->entrytypeid);
        $entrytype = '(<b>'. $result->typename . '</b>) | ';
    }

    // Add authors
    $strauthors = '';
    if(!in_array("authors",$exclude)) {
        $strauthors = "<b>" . get_string("authors","bibliography") . ":</b> ";
        $authors    = bibliography_get_entry_authors($data->id);
        foreach($authors as $author) {
            $strauthors .= $author->firstname . ' ' . $author->lastname . ', ';
        }
        $strauthors = substr($strauthors, 0, -2) . " | ";
    }

    // Add URL?
    $url = '';
    if(!in_array("url",$exclude) && $data->url) {
        if($data->url != 'http://' && $data->url != '') {
            $url = '<br /><b>'.get_string('url_available','bibliography') .':</b> <a href="' . $data->url . '">';
            if(strlen($data->url) >= 30) {
                $strsplit = str_split($data->url,10);
                $url .= $strsplit[0] . '...' . $strsplit[count($strsplit) - 1];
            } else $url .= $data->url;
            $url .= '</a>';
        }
    }
    // Add lower icons?
    $icons = "";
    if(!in_array("no_lower_icons",$exclude))
        $icons = bibliography_print_entry_icons($COURSE, $cm, get_record("bibliography","id",$cm->instance), $data, "delete",'', 'get');

    $result = $entrytype . $strauthors . $string . $url;
    if(strcmp(substr($result,-2),'| ') == 0) $result = substr($result,0,strlen($result) - 3);
    
    return $result . '<br />' . $icons;
}

/**
 * Returns a shortname
 * @param string $name
 * @return string
 */
function generate_shortname($name) {
    global $CFG;
    $shortname = "";
    $array = explode(" ",$name);
    foreach($array as $part) {
        if(strlen($part) < 3 ) {
            $shortname .= (empty($shortname)) ? $part : "-".$part;
        } else {
            $shortname .= (empty($shortname)) ? substr($part,0,3) : "-".substr($part,0,3);
        }
    }
    $n = count_records_sql('select count(*) from '.$CFG->prefix.'bibliography where shortname like "' . $shortname .'%"');
    if($n > 0) return $shortname . "" . $n;
    return $shortname;
}


?>